"""
    Class to write a Bufr file for Argo data in a NetCDF file
"""

from bitstring import BitArray, Bits
from bufr.BufrFXYString import BufrFXYString
from bufr.BufrFXYInt import BufrFXYInt
from bufr.BufrFXYDouble import BufrFXYDouble
from bufr.BufrFXYDoubleReplication import BufrFXYDoubleReplication
from bufr.BufrFXYIntReplication import BufrFXYIntReplication
from bufr.BufrFXYTemplate import BufrFXYTemplate
from bufr.BufrFXYDelayedReplication import BufrFXYDelayedReplication
from bufr.BufrBitArray import BufrBitArray
from bufr.BufrEncoder import BufrEncoder
from channels.Channel import Channel
from datetime import datetime
import pdb

class ArgoNetCDFBufrEncoder(BufrEncoder):
    """Class to write a Bufr file for Argo data in a NetCDF file"""

    def __init__(self):
        super(ArgoNetCDFBufrEncoder, self).__init__()

        #pdb.set_trace()
        self.addDataDescription("TemplateArgo", BufrFXYTemplate(3, 15, 3))

        self.addDataDescription("WMOMarineObservingPlatform",BufrFXYInt(0, 1, 87, 23, True))
        self.addDataDescription("ObservingPlatformManufacturesModel",BufrFXYString(0, 1, 85, 20, True))
        self.addDataDescription("ObservingPlatformManufacturesSerialNumber",BufrFXYString(0, 1, 86, 32, True))
        self.addDataDescription("BuoyType",BufrFXYInt(0, 2, 36, 2, True))
        self.addDataDescription("DataCollectionAndOrLocationSystem",BufrFXYInt(0, 2, 148, 5, True))
        self.addDataDescription("DataTypeOfBuoy",BufrFXYInt(0, 2, 149, 6, True))
        self.addDataDescription("FloatCycleNumber",BufrFXYInt(0, 22, 55, 10, True))
        self.addDataDescription("DirectionOfProfile",BufrFXYInt(0, 22, 56, 2, True))
        self.addDataDescription("InstrumentTypeForWaterTemperatureProfileMeasurement",BufrFXYInt(0, 22, 67, 10, True))

        #self.addDataDescription("TemplateDate", BufrFXYTemplate(3, 1, 11))
        self.addDataDescription("Year",BufrFXYInt(0, 4, 1, 12, True))
        self.addDataDescription("Month",BufrFXYInt(0, 4, 2, 4, True))
        self.addDataDescription("Day",BufrFXYInt(0, 4, 3, 6, True))

        #self.addDataDescription("TemplateTime", BufrFXYTemplate(3, 1, 12))
        self.addDataDescription("Hour",BufrFXYInt(0, 4, 4, 5, True))
        self.addDataDescription("Minute",BufrFXYInt(0, 4, 5, 6, True))

        #self.addDataDescription("TemplatePosition", BufrFXYTemplate(3, 1, 21))
        self.addDataDescription("LatitudeCoarse",BufrFXYDouble(0, 5, 1, 5.0, 9000000.0, 25, True))
        self.addDataDescription("LongitudeCoarse",BufrFXYDouble(0, 6, 1, 5.0, 18000000.0, 26, True))

        self.addDataDescription("Position_QualifierForGTSPPQualityClass",BufrFXYInt(0, 8, 80, 6, True))
        self.addDataDescription("Position_GlobalGTSPPQualityClass",BufrFXYInt(0, 33, 50, 4, True))

        # replication descriptors
        self.addDataDescription("DelayedReplication", BufrFXYDelayedReplication(9, True)) # replication of next 9 items
        self.addDataDescription("ExtendedDelayedReplication",BufrFXYInt(0, 31, 2, 16, True))   # number of replications (i.e. levels)

        # replicated section
        self.addDataDescription("WaterPressure",BufrFXYDoubleReplication(0, 7, 65, -3.0, 0.0, 17, True))
        self.addDataDescription("WaterPressure_QualifierForGTSPPQualityClass",BufrFXYIntReplication(0, 8, 80, 6, True))
        self.addDataDescription("WaterPressure_GlobalGTSPPQualityClass",BufrFXYIntReplication(0, 33, 50, 4, True))

        self.addDataDescription("SeaWaterTemperature",BufrFXYDoubleReplication(0, 22, 45, 3.0, 0.0, 19, True))
        self.addDataDescription("SeaWaterTemperature_QualifierForGTSPPQualityClass",BufrFXYIntReplication(0, 8, 80, 6, True))    # water temperature at a level = 11
        self.addDataDescription("SeaWaterTemperature_GlobalGTSPPQualityClass",BufrFXYIntReplication(0, 33, 50, 4, True))

        self.addDataDescription("Salinity",BufrFXYDoubleReplication(0, 22, 64, 3.0, 0.0, 17, True))
        self.addDataDescription("Salinity_QualifierForGTSPPQualityClass",BufrFXYIntReplication(0, 8, 80, 6, True))   # salinity at a level = 12
        self.addDataDescription("Salinity_GlobalGTSPPQualityClass",BufrFXYIntReplication(0, 33, 50, 4, True))

        pass

    # this is called by the base class when it needs the data values
    def setData(self):

        # map the Bufr data description to the Argo channel name

        # meta data
        self.getDataDescription("WMOMarineObservingPlatform").setValue(self.channels["PLATFORM_NUMBER"].value)
        self.getDataDescription("ObservingPlatformManufacturesModel").setValue(self.channels["PLATFORM_TYPE"].value)
        self.getDataDescription("ObservingPlatformManufacturesSerialNumber").setValue(self.channels["FLOAT_SERIAL_NO"].value)
        self.getDataDescription("BuoyType").setValue(2) # sub-surface float = 2

        positioningSystem = self.channels["POSITIONING_SYSTEM"].value
        positioningSystems = {"ARGOS": 1, "GPS": 2, "IRIDIUM": 3}
        self.getDataDescription("DataCollectionAndOrLocationSystem").setValue(positioningSystems.get(positioningSystem, 7)) # default to 7

        self.getDataDescription("DataTypeOfBuoy").setValue(26) # Sub-surface ARGO float = 26
        self.getDataDescription("FloatCycleNumber").setValue(self.channels["CYCLE_NUMBER"].value)

        profileDirection = self.channels["DIRECTION"].value
        profileDirections = {"A": 0, "D": 1}
        self.getDataDescription("DirectionOfProfile").setValue(profileDirections.get(profileDirection, 3)) # default to 3

        self.getDataDescription("InstrumentTypeForWaterTemperatureProfileMeasurement").setValue(self.channels["WMO_INST_TYPE"].value)

        # date/time
        measurementDateTime = self.getMeasurementTime()
        self.getDataDescription("Year").setValue(measurementDateTime.year)
        self.getDataDescription("Month").setValue(measurementDateTime.month)
        self.getDataDescription("Day").setValue(measurementDateTime.day)
        self.getDataDescription("Hour").setValue(measurementDateTime.hour)
        self.getDataDescription("Minute").setValue(measurementDateTime.minute)

        # position
        self.getDataDescription("LatitudeCoarse").setValue(self.channels["LATITUDE"].value)
        self.getDataDescription("LongitudeCoarse").setValue(self.channels["LONGITUDE"].value)
        self.getDataDescription("Position_QualifierForGTSPPQualityClass").setValue(20)  # position = 20
        self.getDataDescription("Position_GlobalGTSPPQualityClass").setValue(self.channels["POSITION_QC"].value)

        # replicated section
        levels = self.channels["N_LEVELS"].value
        self.getDataDescription("ExtendedDelayedReplication").setValue(levels)

        level = 0
        self.getDataDescription("WaterPressure").setValue(self.channels["PRES"].value[level])  # Pa
        self.getDataDescription("WaterPressure_QualifierForGTSPPQualityClass").setValue([10] * levels)  # water pressure at a level = 10
        self.getDataDescription("WaterPressure_GlobalGTSPPQualityClass").setValue(self.channels["PRES_QC"].value[level])

        self.getDataDescription("SeaWaterTemperature").setValue(self.channels["TEMP"].value[level])  # degC
        self.getDataDescription("SeaWaterTemperature_QualifierForGTSPPQualityClass").setValue([11] * levels)  # water temperature at a level = 11
        self.getDataDescription("SeaWaterTemperature_GlobalGTSPPQualityClass").setValue(self.channels["TEMP_QC"].value[level])

        self.getDataDescription("Salinity").setValue(self.channels["PSAL"].value[level])  # degC
        self.getDataDescription("Salinity_QualifierForGTSPPQualityClass").setValue([12] * levels)  # salinity at a level = 12
        self.getDataDescription("Salinity_GlobalGTSPPQualityClass").setValue(self.channels["PSAL_QC"].value[level])
        pass

    #def getMeasurementTime(self) -> datetime:
    def getMeasurementTime(self):
        # date/time
        julianDay1950 = self.channels["JULD"].value
        julianDay1970 = (julianDay1950 - 7305) * 86400  # days between 1950 and 1970
        measurementDateTime = datetime.utcfromtimestamp(julianDay1970)
        return measurementDateTime
        pass

    # get the Bufr data from the channel data passed
    #def getMessage(self, argoChannels: dict) -> BitArray:
    def getMessage(self, argoChannels):
        self.channels = argoChannels
        buffer = BufrBitArray()
        measurementDateTime = self.getMeasurementTime()
        self.writeMessage(buffer, measurementDateTime, 74) # Exeter = 74
        return buffer
        pass

