import unittest
import re
import numpy as np
from netCDF4 import Dataset, chartostring
from channels.Channel import Channel
from bitstring import BitStream, BitString
import logging
import pdb

class NetCDFDecoder(object):
    """Class to read a NetCDF file from an Argo sensor and return a map of channels"""

    def __init__(self):
        pass

    #def loadFile(self, fileName: str):
    def loadFile(self, fileName):
        self.rootgrp = Dataset(fileName, "a")
        # create an empty dictionary
        self.channels = {}

    #def decode(self) -> dict:
    def decode(self):
        # decode each variable from the NetCDF file and add its data to the channels list

        # clear dictionary
        self.channels.clear

        # channels

        # headers and useful info
        self.decodeCharChannelToInteger("PLATFORM_NUMBER")            # WMO number
        self.decodeCharChannelToString("FLOAT_SERIAL_NO", "", True)   # serial number of float (allow this info to be missing)
        self.decodeCharChannelToString("POSITIONING_SYSTEM")          # positioning system - Argo reference table 9
        self.decodeCharChannelToString("PLATFORM_TYPE", "", True)     # type of platform - Argo reference table 23 (allow this info to be missing)
        self.decodeIntegerChannel("CYCLE_NUMBER")                     # cycle number
        self.decodeCharChannelToString("DIRECTION")                   # direction of profile A : ascending profile D : descending profile
        self.decodeCharChannelToInteger("WMO_INST_TYPE")              # instrument type
        self.decodeFloatChannel("JULD")                               # Julian day of measurement (1 = a day, starting at 1970)
        self.decodeFloatChannel("LONGITUDE")                          # longitude
        self.decodeFloatChannel("LATITUDE")                           # latitude
        self.decodeCharChannelToInteger("POSITION_QC")                # quality of position measurement
        self.decodeDimension("N_LEVELS")                              # number of levels in profiles
        self.decodeDimension("N_PROF")                                # number of profiles

        # data
        self.decodeCharChannelToString("DATA_MODE")
        dataChannelNames = ["PRES", "TEMP", "PSAL"]
        dataMode = self.channels["DATA_MODE"].value
        if (dataMode=="D" or dataMode=="A"):
            for channelName in dataChannelNames:
                [scale, offset] = self.getScaleOffsetForChannel(channelName)
                self.decodeFloatsChannel(channelName + "_ADJUSTED", scale, offset, channelName)
                self.decodeIntsChannel(channelName + "_ADJUSTED_QC", channelName + "_QC")
        else:
            for channelName in dataChannelNames:
                [scale, offset] = self.getScaleOffsetForChannel(channelName)
                self.decodeFloatsChannel(channelName, scale, offset)
                self.decodeIntsChannel(channelName + "_QC")

        # more than one profile?
        numProfiles = self.channels["N_PROF"].value
        if (numProfiles>1):
            # need to load the bio file, check dimensions, and read??
            #//??
            pass

        return self.channels

    # add units here using the units from the ARGO file
    #def getScaleOffsetForChannel(self, channelName: str) -> [float, float]:
    def getScaleOffsetForChannel(self, channelName):
        # conversion factors
        DBAR_TO_PASCAL = 10000.0
        HPASCAL_TO_PASCAL = 100.0
        DEGC_TO_KELVIN = 273.15
        # mapping from ARGO units to scale/offset
        pressureUnitsConvertersScale = {"decibar": DBAR_TO_PASCAL, "hectopascal": HPASCAL_TO_PASCAL}
        temperatureUnitsConvertersOffset = {"degree_Celsius": DEGC_TO_KELVIN}
        # mapping from channel name to unit conversions
        channelUnitsScale = {"PRES": pressureUnitsConvertersScale, "TEMP": {}, "PSAL": {}}
        channelUnitsOffset = {"PRES": {}, "TEMP": temperatureUnitsConvertersOffset, "PSAL": {}}
        unitsConvertersScale = channelUnitsScale[channelName]
        unitsConvertersOffset = channelUnitsOffset[channelName]
        units = self.getChannelUnits(channelName)
        scale = unitsConvertersScale.get(units, 1.0)    # default to make no change
        offset = unitsConvertersOffset.get(units, 0.0)
        return [scale, offset]
        pass

    #def getGey(self, key: str, name: str):
    def getGey(self, key, name):
        return name if (key=="") else key
        pass

    #def decodeCharChannelToInteger(self, name: str, key: str = ""):
    def decodeCharChannelToInteger(self, name, key = ""):
        v = self.rootgrp.variables[name]
        vstr = v[0]
        if type(vstr) is np.ma.core.MaskedArray:
            vstr = chartostring(vstr)
            vstr = vstr.tostring()
        else:
            vstr = vstr.decode("US-ASCII")
        vmissing = v._FillValue
        vint = int(vstr)
        logging.info("CDF data " + name + " = " + str(vint))
        self.channels[self.getGey(key, name)] = Channel(name, vint, vmissing, v)

    #def decodeCharChannelToString(self, name: str, key: str = ""):
    def decodeCharChannelToString(self, name, key = "", allowMissing = False):
        if name in self.rootgrp.variables or not allowMissing:
          v = self.rootgrp.variables[name]
          vstr = v[0]
          if type(vstr) is np.ma.core.MaskedArray:
              vstr = chartostring(vstr).tostring()
          vstr = vstr.decode("US-ASCII")
          vstr = vstr.encode()
          vstr = vstr.strip(" ")
          vmissing = v._FillValue
          vmissing = vmissing.decode("US-ASCII")
          vstr = re.sub(vmissing, '', vstr)
        else:
          vstr = ""
          vmissing = ""
          v = ""
        logging.info("CDF data " + name + " = " + vstr)
        self.channels[self.getGey(key, name)] = Channel(name, vstr, vmissing, v)

    #def decodeIntegerChannel(self, name: str, key: str = ""):
    def decodeIntegerChannel(self, name, key = ""):
        v = self.rootgrp.variables[name]
        vint = int(v[0])
        vmissing = v._FillValue
        if (vint == vmissing):
            vint = None
        logging.info("CDF data " + name + " = " + str(vint))
        self.channels[self.getGey(key, name)] = Channel(name, vint, vmissing, v)

    #def decodeFloatChannel(self, name: str, key: str = ""):
    def decodeFloatChannel(self, name, key = ""):
        v = self.rootgrp.variables[name]
        vfloat = float(v[0])
        vmissing = v._FillValue
        if (vfloat == vmissing):
            vfloat = None
        logging.info("CDF data " + name + " = " + str(vfloat))
        self.channels[self.getGey(key, name)] = Channel(name, vfloat, vmissing, v)

    #def decodeDimension(self, name: str, key: str = ""):
    def decodeDimension(self, name, key = ""):
        v = self.rootgrp.dimensions[name]
        #vint = v.size
        vint = len(v)
        logging.info("CDF dimension " + name + " = " + str(vint))
        self.channels[self.getGey(key, name)] = Channel(name, vint, None, v)

    #def decodeFloatsChannel(self, name: str, scaleFactor: float, offset: float, key: str = ""):
    def decodeFloatsChannel(self, name, scaleFactor, offset, key = ""):
        v = self.rootgrp.variables[name]
        vfloats = (v[:] * scaleFactor) + offset
        vmissing = v._FillValue
        if type(vfloats) is np.ma.core.MaskedArray:
            vfloats._sharedmask = False # this will be the default behaviour with Numpy 1.12
        vfloats[vfloats==vmissing] = None
        self.channels[self.getGey(key, name)] = Channel(name, vfloats, vmissing, v)
        pass

    #def decodeIntsChannel(self, name: str, key: str = ""):
    def decodeIntsChannel(self, name, key = ""):
        v = self.rootgrp.variables[name]
        vints = v[:]
        vmissing = v._FillValue
        if type(vints) is np.ma.core.MaskedArray:
            vints._sharedmask = False   # this will be the default behaviour with Numpy 1.12
        vints[vints==vmissing] = None
        self.channels[self.getGey(key, name)] = Channel(name, vints, vmissing, v)
        pass

    #def decodeBitsChannel(self, name: str, key: str = ""):
    def decodeBitsChannel(self, name, key = ""):
        v = self.rootgrp.variables[name]
        vstr = chartostring(v[:]).tostring()
        vstr = vstr.decode("US-ASCII")
        # 1 is good, everything else is bad
        vstr = re.sub('[023456789]', '0', vstr)
        bits = BitString(bin="0b"+vstr)
        vmissing = v._FillValue
        self.channels[self.getGey(key, name)] = Channel(name, bits, vmissing, v)   # add as a sequence of bits, which is more efficient
        pass

    #def getChannelUnits(self, name: str) -> str:
    def getChannelUnits(self, name):
        v = self.rootgrp.variables[name]
        return v.units
        pass
